/*
 * Code for controlling GPIOs. This is meant to be an extension
 * to the simple PORTx/PINx/DDRx Interface.
 * You simply can specify a various number of GPIOs that are
 * mapped to randomly specified pins on the avr. 
 * 
 * Author: Simon Kueppers
 * Email: simon.kueppers@web.de
 * Homepage: http://klinkerstein.m-faq.de
 * 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 Copyright 2008 Simon Kueppers
 * */
 
// modified D. Festing 18Apr2009 for use on the JEDMicro Dev board
 

#include "Gpio.h"
#include <avr/pgmspace.h>
#include <avr/io.h>

struct tGpio
{
	volatile uint8_t* pPin;
	volatile uint8_t* pPort;
	uint8_t nBit;
};

// PB0 now used for DAC select, but leave it in the struct OR
// you have to re-number the HTML file!
// Also, don't think PA0, PA1, PA2, PA3 and PA4 should be in here
struct tGpio PROGMEM g_Gpio[] =
{
   {&PINA, &PORTA, PA0},
	{&PINA, &PORTA, PA1},
	{&PINA, &PORTA, PA2},
	{&PINA, &PORTA, PA3},
	{&PINA, &PORTA, PA4},
	{&PINA, &PORTA, PA5},
	{&PINA, &PORTA, PA6},
	{&PINA, &PORTA, PA7},
	{&PINB, &PORTB, PB0},
	{&PINB, &PORTB, PB3},
   {&PINC, &PORTC, PC2},
	{&PINC, &PORTC, PC3},
	{&PINC, &PORTC, PC4},
	{&PINC, &PORTC, PC5},
   {&PINC, &PORTC, PC6},
	{&PINC, &PORTC, PC7},
   {&PIND, &PORTD, PD2},
	{&PIND, &PORTD, PD3},
	{&PIND, &PORTD, PD4},
	{&PIND, &PORTD, PD5},
	{&PIND, &PORTD, PD6},
	{&PIND, &PORTD, PD7}
};

uint8_t GpioGetPin(uint8_t nGpio)
{
	volatile uint8_t* pPin =
			(volatile uint8_t*) pgm_read_word(&(g_Gpio[nGpio].pPin));
	uint8_t nBit = pgm_read_byte(&(g_Gpio[nGpio].nBit));

	return (*pPin >> nBit) & 1;
}


uint8_t GpioGetPort(uint8_t nGpio)
{
	volatile uint8_t* pPort =
			(volatile uint8_t*) pgm_read_word(&(g_Gpio[nGpio].pPort));
	uint8_t nBit = pgm_read_byte(&(g_Gpio[nGpio].nBit));

	return (*pPort >> nBit) & 1;
}


void GpioSetPort(	uint8_t nGpio,
					uint8_t nState)
{
	volatile uint8_t* pPort =
			(volatile uint8_t*) pgm_read_word(&(g_Gpio[nGpio].pPort));
	uint8_t nBit = pgm_read_byte(&(g_Gpio[nGpio].nBit));

	if (nState)
		*pPort |= (1<<nBit);
	else
		*pPort &= ~(1<<nBit);
}
