/*
 * Main File including the main-routine with initializations-
 * and stack-routines.
 * 
 * Author: Simon Kueppers
 * Email: simon.kueppers@web.de
 * Homepage: http://klinkerstein.m-faq.de
 * */


// modified D. Festing 22 July 2010

// Pigeon Bay webserver project using the JedMicro dev board
// changes to ENC28J60.h for the Atmega32
// F_CPU changed to 12.5MHz
// the ENC28J60 interrupt pin is not used.  Connecting this output to PC1,
// which is not an interrupt pin, is not correct!
// SPI_PRESCALAR = 16, which is 400kHz.  Originally SPI_PRESCALAR = 2
// DAC didn't work at 4, so backed off 2 steps.  Still plenty fast for Ethernet. 
// time to go around the loop = 100us
// debounce any switches on input ports


// ToDo:
// put default DAC values into EEPROM and read into DAC_data at startup
// and/or save DAC_data to EEPROM every time it changes
// enable BOD
// disable UDP saves about 2% of Program and Data space
// maybe use an external control on the web page to reset the sendmail_flag
// after a post has been sent the unit does not send the updated page back.
// original code DID


#include "project.h"
#include "PB_io.h"
#include "Enc28j60.h"
#include "dac.h"
#include "uip.h"
#include "uip_arp.h"
#include "uip_TcpAppHub.h"
#include "smtp.h"


#define STACK_CANARY 0xc5


// stack monitoring variables
extern uint8_t _end;
extern uint8_t __stack;

uint8_t sendmail_flag = 0;
static uint8_t g_nPrescaler = 100;
static volatile struct
{
	uint8_t fPeriodic :1;
} g_nFlags = { 0 };


void StackPaint(void) __attribute__ ((naked)) __attribute__ ((section (".init1")));
void StackPaint(void);


int main(void)
{
	uint8_t dac_value = 0;
   uint8_t dac_address = 0;
   uip_ipaddr_t IpAddr;
// stack monitoring pointer
   const uint8_t *p = &_end;
   uint16_t c = 0;
   int8_t i = 0;
	uint16_t counter = 0; // debounce counter

   initialise_hardware();

// SS is set up in Enc28j60.c This needs to be set to output BEFORE
// using the ENC28J60 OR the DAC!
	Enc28j60Init();

// initialise DAC values
   DAC_DESELECT(); // ensure DAC_enable is low
   
	dac_value = 0;
   
// initialise DAC outputs on boot-up to zero in case power is lost momentarily,
// i.e. the DAC outputs haven't had time to drop to zero volts        
   for(i = 1; i < 7; i++)
   {
      load_DAC(i, dac_value);
   }

// load default DAC values
   dac_address = 1;
   dac_value = 127;
   
   load_DAC(dac_address, dac_value);
   
   dac_address = 2;
   dac_value = 127;
 
   load_DAC(dac_address, dac_value);


   uip_arp_init();
   uip_init();
   uip_TcpAppHubInit();

// configure SMTP host
   uip_ipaddr(IpAddr, 203, 97, 33, 211);
   smtp_configure("PigeonBay", IpAddr);

	Enc28j60SetClockPrescaler(2);

	TCCR1B |= (1<<WGM12) | (3<<CS10);
	OCR1A = 19531; //12500000 / 64 / 19531,25 = 10Hz (100ms)
	TIMSK |= (1<<OCIE1A);

  
	uip_ipaddr(IpAddr, 192, 168, 1, 4);
	uip_sethostaddr(IpAddr);
   uip_ipaddr(IpAddr, 192, 168, 1, 1);
   uip_setdraddr(IpAddr);
	uip_ipaddr(IpAddr, 255, 255, 255, 0);
	uip_setnetmask(IpAddr);


	sei();


	FOREVER
	{
		if ((PIND & (uint8_t)(1<<2)) == 0 && sendmail_flag == 0)
      {
         counter++;

         if (counter == 1000)
         {
            sendmail_flag = 1; 
			   counter = 0;

            SMTP_SEND("dave_festing@hotmail.com", NULL, "dave_festing@clear.net.nz","PigeonBay Alert","Downstair's door open\r\n");
			}	
      }

		if ((PIND & (uint8_t)(1<<2)) != 0 && sendmail_flag == 1)
      {
         counter++;
		
         if (counter == 1000)
         {
      	   sendmail_flag = 0; 
			   counter = 0;
			}	
      }

      uip_len = Enc28j60Receive(uip_buf);

		if (uip_len > 0)
		{
			if (((struct uip_eth_hdr *)&uip_buf[0])->type
					== htons(UIP_ETHTYPE_IP))
			{
            uip_input();

				if (uip_len > 0)
				{
					uip_arp_out();
					Enc28j60CopyPacket(uip_buf, uip_len);
					Enc28j60Send();
				}
			}
			else if (((struct uip_eth_hdr *)&uip_buf[0])->type
					== htons(UIP_ETHTYPE_ARP))
			{
				uip_arp_arpin();

				if (uip_len > 0)
				{
					Enc28j60CopyPacket(uip_buf, uip_len);
					Enc28j60Send();
				}
			}
		}

		if (g_nFlags.fPeriodic)
		{
         cli();
			g_nFlags.fPeriodic = 0;
			sei();

			int16_t i = UIP_CONNS;

			while (i)
			{
				i--;
				uip_periodic(i);
				if (uip_len > 0)
				{
					uip_arp_out();
					Enc28j60CopyPacket(uip_buf, uip_len);
					Enc28j60Send();
				}
			}

			g_nPrescaler--;

			if (g_nPrescaler == 0)
			{
			// stack count
            while(*p == STACK_CANARY && p <= &__stack)
            {
               p++;
               c++;
            }
         // we are running low on RAM
            if(c < 100)
            {
               PORTC |= (1<<PC2);

               while(1);
            }

         // every 10 seconds
				uip_arp_timer();
				g_nPrescaler = 100;
			}
		}
	}

	return 0;
}


void StackPaint(void)
{
   uint8_t *p = &_end;

   while(p <= &__stack)
   {
      *p = STACK_CANARY;
      p++;
   }
}


ISR(TIMER1_COMPA_vect)
{
	g_nFlags.fPeriodic = 1;
}

