/*
 * Code file containing the HTTP Daemon module.
 * It supports GET (not completely implemented) and POST Request methods
 * 
 * Author: Simon Kueppers
 * Email: simon.kueppers@web.de
 * Homepage: http://klinkerstein.m-faq.de
 * 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 Copyright 2008 Simon Kueppers
 * */

// modified D. Festing 18 May 2009 to allow placing int values into the HTML file
// and to talk to a 6 channel DAC


#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "HttpD.h"
#include "../uip/uip.h"
#include "../../Hardware/Gpio.h"
#include "../../Hardware/adc.h"
#include "../../Hardware/dac.h"
#include <string.h>
#include <stdlib.h>


#define STATE_CONNECTED		0	//Client has just connected
#define STATE_GET_G			1	//G
#define STATE_GET_GE		   2	//GE
#define STATE_GET_GET		3	//GET
#define STATE_POST_P		   4	//P
#define STATE_POST_PO		5	//PO
#define STATE_POST_POS		6	//POS
#define STATE_POST_POST		7	//POST
#define STATE_GOTGET		   8	//Client just sent a GET request
#define STATE_GOTPOST		9	//Client just sent a POST request
#define STATE_PARSEPOST		10	//we are currently parsing the client's POST-data
#define STATE_SENDHEADER	11	//next we send him the HTTP header
#define STATE_SENDDATA		12	//followed by data

#define PARSE_CMD			   0
#define PARSE_NUM10			1
#define PARSE_NUM1			2
#define PARSE_EQUAL			3
#define PARSE_VAL			   4
#define PARSE_DELIM			5

#define PARSEBYTES			94	// stop parsing POST data after x bytes

/* After each modification of the HTML script below: compile, load the new .hex file and run the
   application.  Then fire up WireShark, Modify and Reload current page to achieve a Post, click on
   POST, Hypertext Transfer Protocol and content length to find the precise number of bytes. Place this
   number above as the new PARSEBYTES and re-compile.
*/   

#define NUMBER_OF_PASSWORD_CHARACTERS 9
// password in HttpDCall()


static const prog_char
		g_HtmlPageDefault[] =
				"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"
				"<html><head>"
				"<title>Pigeon Bay Web Server</title>"
				"<style type='text/css'>"
				".s0 { background-color: Cyan; }"
				".s1 { background-color: LimeGreen; }"
				".s2 { background-color: Red; }"
				"td { text-align: center; }"
            "td { height: 25px; }"
				"</style>"
				"</head>"

            "<body>"
				"<h2>Pigeon Bay Web Server</h2>"

            "<form method=POST action='/'>"

            "<p><b>Password:<input type='password' name='password'></b></p>" 
            
            "<table border='1' cellpadding='1'>"

				"<tr><td colspan='6' width='240'><b>INPUTS</b></td><td colspan='6' width='240'><b>OUTPUTS</b></td></tr>"

				"<tr><td colspan='4' width='160'><b>ADCs</b></td><td colspan='2' width='80'><b>Value</b></td><td colspan='4' width='160'><b>Normal</b></td><td colspan='1' width='40'><b>PIN</b></td><td colspan='1' width='40'><b>State</b></td></tr>"

            "<tr><td colspan='3' width='120'>Batt volts</td><td colspan='1' width='40'>PA0</td><td colspan='2' width='80'><b>%a00</b></td><td colspan='6' width='240'>&nbsp;</td></tr>"

            "<tr><td colspan='3' width='120'>PV current</td><td colspan='1' width='40'>PA1</td><td colspan='2'  width='80'><b>%a01</b></td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PB3</td><td  colspan='1' width='40' class='s%i09'>%i09</td><td colspan='1' width='40' class='s%o09'><input type='text' name='o09' value='%o09' maxlength='1' size='1'></td></tr>"

            "<tr><td colspan='3' width='120'>Outside temp</td><td colspan='1' width='40'>PA2</td><td colspan='2'  width='80'><b>%a02</b></td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD4</td><td  colspan='1' width='40' class='s%i18'>%i18</td><td colspan='1' width='40' class='s%o18'><input type='text' name='o18' value='%o18' maxlength='1' size='1'></td></tr>"            

            "<tr><td colspan='3' width='120'>House temp</td><td colspan='1' width='40'>PA3</td><td colspan='2'  width='80'><b>%a03</b></td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD5</td><td  colspan='1' width='40' class='s%i19'>%i19</td><td colspan='1' width='40' class='s%o19'><input type='text' name='o19' value='%o19' maxlength='1' size='1'></td></tr>"

            "<tr><td colspan='3' width='120'>Tunnel temp</td><td colspan='1' width='40'>PA4</td><td colspan='2'  width='80'><b>%a04</b></td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD7</td><td  colspan='1' width='40' class='s%i21'>%i21</td><td colspan='1' width='40' class='s%o21'><input type='text' name='o21' value='%o21' maxlength='1' size='1'></td></tr>"

//            "<tr><td colspan='12' width='480'></td></tr>"

            "<tr><td colspan='4' width='160'><b>I/P (pullup enabled)</b></td><td colspan='2' width='80'><b>I/P state</b></td><td colspan='4' width='160'><b>FET (inverted)</b></td><td colspan='1' width='40'><b>PIN</b></td><td colspan='1' width='40'><b>State</b></td></tr>"

// can't split the above row into 5 and 1 for the first half
// appears to due to the fact that the PIN column has not been previously split  

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PA5</td><td colspan='2' width='80' class='s%i05'>%i05</td><td colspan='3' width='120'>DO NOT USE</td><td colspan='1' width='40'>PC2</td><td  colspan='1' width='40'class='s%i10'>%i10</td><td colspan='1' width='40' class='s%o10'><input type='text' name='o10' value='%o10' maxlength='1' size='1'></td></tr>"

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PA6</td><td colspan='2' width='80' class='s%i06'>%i06</td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PC3</td><td  colspan='1' width='40'class='s%i11'>%i11</td><td colspan='1' width='40' class='s%o11'><input type='text' name='o11' value='%o11' maxlength='1' size='1'></td></tr>"

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PA7</td><td colspan='2' width='80' class='s%i07'>%i07</td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PC4</td><td  colspan='1' width='40'class='s%i12'>%i12</td><td colspan='1' width='40' class='s%o12'><input type='text' name='o12' value='%o12' maxlength='1' size='1'></td></tr>"   

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD2</td><td colspan='2' width='80' class='s%i16'>%i16</td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PC5</td><td  colspan='1' width='40'class='s%i13'>%i13</td><td colspan='1' width='40' class='s%o13'><input type='text' name='o13' value='%o13' maxlength='1' size='1'></td></tr>"

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD3</td><td colspan='2' width='80' class='s%i17'>%i17</td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PC6</td><td  colspan='1' width='40'class='s%i14'>%i14</td><td colspan='1' width='40' class='s%o14'><input type='text' name='o14' value='%o14' maxlength='1' size='1'></td></tr>"

               "<tr><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PD6</td><td colspan='2' width='80' class='s%i20'>%i20</td><td colspan='3' width='120'>***</td><td colspan='1' width='40'>PC7</td><td  colspan='1' width='40' class='s%i15'>%i15</td><td colspan='1' width='40'  class='s%o15'><input type='text' name='o15' value='%o15' maxlength='1' size='1'></td></tr>"

//               "<tr><td colspan='12' width='480'></td></tr>"

               "<tr><td colspan='3' width='120'><b>Function</b></td><td colspan='1' width='40'><b>Chan</b></td><td colspan='2' width='80'><b>Value</b></td><td colspan='3' width='120'><b>Function</b></td><td colspan='1' width='40'><b>Chan</b></td><td colspan='2' width='80'><b>Value</b></td></tr>"

// note DAC starts at channel 1
               "<tr><td colspan='3' width='120'>Adjust temp</td><td colspan='1' width='40'>1</td><td colspan='2' width='80'><input type='text' name='v01' value='%v01' maxlength='3' size='3'></td><td colspan='3' width='120'>Adjust volts</td><td colspan='1' width='40'>2</td><td colspan='2' width='80'><input type='text' name='v02' value='%v02' maxlength='3' size='3'></td></tr>"

				"</table>"
            "<br>"

            "<input type='submit' value='Modify'>"

				"</form>"
            "</body></html>";

 
static uint16_t CopyStringP(uint8_t** ppBuffer,
							const prog_char* pString)
{
	uint16_t nBytes = 0;
	char Character;

	while ((Character = pgm_read_byte(pString)) != '\0')
	{
		**ppBuffer = Character;
		*ppBuffer = *ppBuffer + 1;
		pString = pString + 1;
		nBytes++;
	}

	return nBytes;
}

static uint16_t CopyValue(	uint8_t** ppBuffer,
							uint32_t nValue)
{
	static PROGMEM int32_t nDecTab[] =
	{
		1000000000, 100000000, 10000000, 1000000, 100000, 10000, 1000, 100, 10,
				1, 0 };

	int32_t nDec;
	uint8_t nCurTabIndex = 0;
	char cNumber;
	uint8_t nBytes = 0;

	while ((nDec = pgm_read_dword(&nDecTab[nCurTabIndex])) != 0)
	{
		for (cNumber='0'; cNumber<'9'; cNumber++)
		{
			int32_t nTmp = nValue - nDec;

			if (nTmp < 0)
				break;

			nValue = nTmp;
		}

		**ppBuffer = cNumber;
		*ppBuffer = *ppBuffer + 1;
		nBytes++;
		nCurTabIndex++;
	}

	return nBytes;
}

static uint16_t CopyHttpHeader(	uint8_t* pBuffer,
								uint32_t nDataLen)
{
	uint16_t nBytes = 0;

	nBytes += CopyStringP(&pBuffer, PSTR("HTTP/1.1 200 OK"));
	nBytes += CopyStringP(&pBuffer, PSTR("\r\n"));

	nBytes += CopyStringP(&pBuffer, PSTR("Content-Length:"));
	nBytes += CopyValue(&pBuffer, nDataLen);
	nBytes += CopyStringP(&pBuffer, PSTR("\r\n"));

	nBytes += CopyStringP(&pBuffer, PSTR("Content-Type:text/html\r\n"));
	nBytes += CopyStringP(&pBuffer, PSTR("Connection:close\r\n"));
	nBytes += CopyStringP(&pBuffer, PSTR("\r\n"));

	return nBytes;
}


static uint16_t CopyHttpData(uint8_t* pBuffer,
							const prog_void** ppData,
							uint32_t* pDataLeft,
							uint16_t nMaxBytes)
{
	uint16_t nBytesProcessed = 0;
	uint8_t byteToProcess;
	uint8_t nParsedNum;
	uint8_t nParsedMode;
   int8_t i;
// NUMBER_OF_INSERTION_CHARACTERS plus one for '\0'
// plus another for a possible - sign, just to be real safe
   char adc_value[NUMBER_OF_INSERTION_CHARACTERS + 2];
   char dac_value[NUMBER_OF_DAC_CHARACTERS + 1];
// need to be static because there may be multiple calls to CopyHttpData()
   static ADC_data_s ADC_data_get;                       // define the ADC_data_get structure
   static ADC_data_s *ADC_data_get_ptr = &ADC_data_get; // define/declare and initialise a pointer to ADC_data_get    
   static DAC_data_s DAC_data_get;                      // define the DAC_data_get structure
   static DAC_data_s *DAC_data_get_ptr = &DAC_data_get; // define/declare and initialise a pointer to DAC_data_get    
   
   
   AdcDoConversions();
   get_ADC_data(&ADC_data_get);
   get_DAC_data(&DAC_data_get);

	while (nMaxBytes--)
	{
		if (*pDataLeft > 0)
		{
			byteToProcess = pgm_read_byte(*ppData);

			if (byteToProcess == '%')
			{
				*ppData = *ppData + 1;
				*pDataLeft = *pDataLeft - 1;
				nParsedMode = pgm_read_byte(*ppData);
				*ppData = *ppData + 1;
				*pDataLeft = *pDataLeft - 1;
				nParsedNum = (pgm_read_byte(*ppData) - '0') * 10;
				*ppData = *ppData + 1;
				*pDataLeft = *pDataLeft - 1;
				nParsedNum += (pgm_read_byte(*ppData) - '0') * 1;
				
            if (nParsedMode == 'i') // input port
				{
               *pBuffer = GpioGetPin(nParsedNum) + '0';
            }
				else if (nParsedMode == 'o') // output port
				{
               *pBuffer = GpioGetPort(nParsedNum) + '0';
            }
            else if (nParsedMode == 'v') // DAC address and value
				{
            // test for nMaxBytes, bail out if not enough room 
               if(nMaxBytes < NUMBER_OF_DAC_CHARACTERS)
               {
               // rewind both so that next time around we are
               // looking at the % character   
                  *ppData = *ppData - 3;
                  *pDataLeft = *pDataLeft +3;

                  break;
               }   
 
            // accesss a specific dac_value in the DAC_data struct
               switch (nParsedNum)
               {
                  case 0:  break;

                  case 1:  memcpy((char*)dac_value, (char*)DAC_data_get_ptr->DAC_1, sizeof dac_value);

                           break;

                  case 2:  memcpy((char*)dac_value, (char*)DAC_data_get_ptr->DAC_2, sizeof dac_value);

                           break;

                  case 3:  memcpy((char*)dac_value, (char*)DAC_data_get_ptr->DAC_3, sizeof dac_value);

                           break;

                  case 4:  memcpy((char*)dac_value, (char*)DAC_data_get_ptr->DAC_4, sizeof dac_value);

                           break;

                  default: break;
               }            

               for(i = 0; i < DAC_data_get_ptr->count[nParsedNum] - 1; i++)
               {                                                                                                    
                  *pBuffer = dac_value[i];

                  pBuffer++;
                  nMaxBytes--;  
                  nBytesProcessed++;     
               }

               *pBuffer = dac_value[DAC_data_get_ptr->count[nParsedNum] - 1]; // the last character
            }
            else if (nParsedMode == 'a')
            {
            // test for nMaxBytes, bail out if not enough room 
               if(nMaxBytes < NUMBER_OF_INSERTION_CHARACTERS)
               {
               // rewind both so that next time around we are
               // looking at the % character   
                  *ppData = *ppData - 3;
                  *pDataLeft = *pDataLeft +3;

                  break;
               }

            // accesss a specific adc_value in the ADC_data struct
               switch (nParsedNum)
               {
                  case 0:  memcpy((char*)adc_value, (char*)ADC_data_get_ptr->ADC_0, sizeof adc_value);
                           break;
                           
                  case 1:  memcpy((char*)adc_value, (char*)ADC_data_get_ptr->ADC_1, sizeof adc_value);
                           break;
                           
                  case 2:  memcpy((char*)adc_value, (char*)ADC_data_get_ptr->ADC_2, sizeof adc_value);
                           break;
                           
                  case 3:  memcpy((char*)adc_value, (char*)ADC_data_get_ptr->ADC_3, sizeof adc_value);
                           break;
                           
                  case 4:  memcpy((char*)adc_value, (char*)ADC_data_get_ptr->ADC_4, sizeof adc_value);
                           break;
                           
                  default: break;
               }            

               for(i = 0; i < ADC_data_get_ptr->count[nParsedNum] - 1; i++)
               {                                                                                                    
                  *pBuffer = adc_value[i];

                  pBuffer++;
                  nMaxBytes--;  
                  nBytesProcessed++;     
               }

               *pBuffer = adc_value[ADC_data_get_ptr->count[nParsedNum] - 1]; // the last character
            } 
         }
			else
			{
            *pBuffer = byteToProcess;
         }

			*ppData = *ppData + 1;
			*pDataLeft = *pDataLeft - 1;
			pBuffer++;
			nBytesProcessed++;
		}
		else
      {
         break;
      }
	}

	return nBytesProcessed;
}

void HttpDInit()
{
	//Start listening on our port
	uip_listen(HTONS(PORT_HTTPD));
}

void HttpDCall(	uint8_t* pBuffer,
				uint16_t nBytes,
				struct tHttpD* pSocket)
{
	uint16_t nBufSize;
   char temp_buf[3 + 1]; // to hold DAC value
   uint8_t address;
   uint16_t value; // so that we can test for values greater than 255
   uint8_t password_state;

   const char password[] = "*********";    

 
	if (uip_connected())
	{
		//Initialize this connection
		pSocket->pData = g_HtmlPageDefault;
		pSocket->nDataLeft = sizeof(g_HtmlPageDefault) - 1;
		pSocket->nNewlines = 0;
		pSocket->nState = STATE_CONNECTED;
		pSocket->nPrevBytes = 0xFFFF;
	}
	else if (uip_newdata() || uip_acked())
	{
		if (pSocket->nState == STATE_CONNECTED)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'G')
				pSocket->nState = STATE_GET_G;
			else if (*pBuffer == 'P')
				pSocket->nState = STATE_POST_P;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_GET_G)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'E')
				pSocket->nState = STATE_GET_GE;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_GET_GE)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'T')
				pSocket->nState = STATE_GET_GET;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_GET_GET)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == ' ')
				pSocket->nState = STATE_GOTGET;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_POST_P)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'O')
				pSocket->nState = STATE_POST_PO;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_POST_PO)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'S')
				pSocket->nState = STATE_POST_POS;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_POST_POS)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == 'T')
				pSocket->nState = STATE_POST_POST;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_POST_POST)
		{
			if (nBytes == 0)
				return;
			if (*pBuffer == ' ')
				pSocket->nState = STATE_GOTPOST;
			nBytes--;
			pBuffer++;
		}

		if (pSocket->nState == STATE_GOTPOST || pSocket->nState == STATE_GOTGET)
		{
			//Search for \r\n\r\n
			while (nBytes != 0)
			{
				if (*pBuffer == '\n')
				{
					pSocket->nNewlines++;
				}
				else if (*pBuffer == '\r')
				{
				}
				else
				{
					pSocket->nNewlines = 0;
				}

				pBuffer++;
				nBytes--;

				if (pSocket->nNewlines == 2)
				{
					//beginning found.
					if (pSocket->nState == STATE_GOTPOST)
					{
						//Initialize Parsing variables
						pSocket->nParseLeft = PARSEBYTES;
						pSocket->ParseState = PARSE_CMD;
						//start parsing
						pSocket->nState = STATE_PARSEPOST;
					}
					else if (pSocket->nState == STATE_GOTGET)
					{
						pSocket->nState = STATE_SENDHEADER;
					}
					break;
				}
			}
		}

		if (pSocket->nState == STATE_PARSEPOST)
		{
			password_state = 0;
         
         while (nBytes--)
			{
         // find the password
            if(password_state != 1)
            {
               if (strncmp(password, (char*)pBuffer, NUMBER_OF_PASSWORD_CHARACTERS) == 0)
               {
                  password_state = 1;

               // minus 1 because the loop takes care of the last character 
                  nBytes = nBytes - (NUMBER_OF_PASSWORD_CHARACTERS - 1);
                  pSocket->nParseLeft =  pSocket->nParseLeft - (NUMBER_OF_PASSWORD_CHARACTERS);
                  pBuffer = pBuffer + (NUMBER_OF_PASSWORD_CHARACTERS);

               // ignore delimiter between password and first command letter
                  nBytes--; 
                  pSocket->nParseLeft--;
                  pBuffer++;
               }
               else
               {
                  pSocket->nParseLeft--;
                  pBuffer++;
               }   
            } 
            else if (password_state == 1)
            {
               if (pSocket->ParseState == PARSE_CMD)
               {
                  pSocket->ParseCmd = *pBuffer;
                  pSocket->ParseState = PARSE_NUM10;
               }
               else if (pSocket->ParseState == PARSE_NUM10)
               {
                  pSocket->ParseNum = (*pBuffer - '0') * 10;
                  pSocket->ParseState = PARSE_NUM1;
               }
               else if (pSocket->ParseState == PARSE_NUM1)
               {
                  pSocket->ParseNum += (*pBuffer - '0');
                  pSocket->ParseState = PARSE_EQUAL;
               }
               else if (pSocket->ParseState == PARSE_EQUAL)
               {
                  pSocket->ParseState = PARSE_VAL;
               }
               else if (pSocket->ParseState == PARSE_VAL)
               {
                  if (pSocket->ParseCmd == 'o')
                  {
                     GpioSetPort(pSocket->ParseNum, *pBuffer - '0');
                  }
                  else if (pSocket->ParseCmd == 'v')
                  {
                  // do not appear to need to test for nBytes as was done
                  // in CopyHttpData()
 
                     address = pSocket->ParseNum;

                  // read three ASCII characters representing the byte value into temp_buf
                  // you have to say 025 instead of 25, because content length changes
                  // with number of characters entered 
                     temp_buf[0] = *pBuffer;
                     pBuffer++;
                     nBytes--; 
                     pSocket->nParseLeft--;
                     
                     temp_buf[1] = *pBuffer;
                     pBuffer++;
                     nBytes--;
                     pSocket->nParseLeft--;
                     
                     temp_buf[2] = *pBuffer;

                     value = (atoi(temp_buf));

                  // prevent going over 255
                     if(value > 254)
                     {
                        value = 255;
                     }

                     load_DAC(address, value);
                  }

                  pSocket->ParseState = PARSE_DELIM;
               }
               else if (pSocket->ParseState == PARSE_DELIM)
               {
                  pSocket->ParseState = PARSE_CMD;
               }

               pSocket->nParseLeft--;
               pBuffer++;

               if (pSocket->nParseLeft == 0)
               {
               // finished parsing
                  pSocket->nState = STATE_SENDHEADER;

                  break;
               }
            }   
			}
		}

		if (pSocket->nState == STATE_SENDHEADER)
		{
			uip_send(uip_appdata, CopyHttpHeader(uip_appdata, pSocket->nDataLeft));
			pSocket->nState = STATE_SENDDATA;
			return;
		}

		if (pSocket->nState == STATE_SENDDATA)
		{
			//We have sent the HTML Header or HTML Data previously.
			//Now we send (further) Data depending on the Socket's pData pointer
			//If all data has been sent, we close the connection
			pSocket->nPrevBytes = pSocket->nDataLeft;
			nBufSize
					= CopyHttpData(uip_appdata, &pSocket->pData, &pSocket->nDataLeft, uip_mss());
			pSocket->nPrevBytes -= pSocket->nDataLeft;
			
			
			if (nBufSize == 0)
			{
				//No Data has been copied. Close connection
				uip_close();
			}
			else
			{
				//Else send copied data
				uip_send(uip_appdata, nBufSize);
			}
			return;
		}
	}
	else if (uip_rexmit())
	{
		if (pSocket->nPrevBytes == 0xFFFF)
		{
			/* Send header again */
			uip_send(uip_appdata, CopyHttpHeader(uip_appdata, pSocket->nDataLeft));
		}else
		{
			pSocket->pData -= pSocket->nPrevBytes;
			pSocket->nDataLeft += pSocket->nPrevBytes;
			pSocket->nPrevBytes = pSocket->nDataLeft;
			nBufSize
					= CopyHttpData(uip_appdata, &pSocket->pData, &pSocket->nDataLeft, uip_mss());
			pSocket->nPrevBytes -= pSocket->nDataLeft;
			
			if (nBufSize == 0)
			{
				//No Data has been copied. Close connection
				uip_close();
			}
			else
			{
				//Else send copied data
				uip_send(uip_appdata, nBufSize);
			}
		}
		return;
   }
}
