//*****************************************************************************
//
// File Name   : adc.c  
// Author      : D. Festing
// Created     : 6/5/2009
// Revised     : 
// Version     : 1.0
// Target MCU  : Atmel AVR series
// Editor Tabs	: 3
//
// Description : This file processes all ADC readings and puts them into
//               format suitable for sending out over an ethernet port
//
//*****************************************************************************


// not necessary to turn ADCs on and off to change channels


#include "adc.h"
#include <avr/io.h>
#include <string.h>
#include <stdlib.h>


// ADC #defines   
#define ADC_ON  (ADCSRA |= (1<<ADEN))
#define ADC_OFF (ADCSRA &= ~(1<<ADEN))
   
#define ADC_SELECT_CHANNEL0 do{ADMUX &= ~((1<<MUX3) | (1<<MUX2) | (1<<MUX1) | (1<<MUX0));}while(0)
#define ADC_SELECT_CHANNEL1 do{ADMUX &= ~((1<<MUX3) | (1<<MUX2) | (1<<MUX1)); ADMUX |= (1<<MUX0);}while(0)
#define ADC_SELECT_CHANNEL2 do{ADMUX &= ~((1<<MUX3) | (1<<MUX2) | (1<<MUX0)); ADMUX |= (1<<MUX1);}while(0)
#define ADC_SELECT_CHANNEL3 do{ADMUX &= ~((1<<MUX3) | (1<<MUX2)); ADMUX |= ((1<<MUX0) | (1<<MUX1));}while(0)
#define ADC_SELECT_CHANNEL4 do{ADMUX &= ~((1<<MUX2) | (1<<MUX1)); ADMUX |= (1<<MUX3);}while(0)


static ADC_data_s ADC_data;                   // define the ADC_data structure
static ADC_data_s *ADC_data_ptr = &ADC_data; // define/declare and initialise a pointer to ADC_data    


static uint16_t process_temperature_measurement(void);
static uint16_t process_voltage_measurement(void);


void adc_init(void)
{
   uint8_t prescale = 0;
   uint8_t div = 0;

   
// ADC initialisation, finds closest division ratio to give < 200kHz
   ADMUX |= (1<<REFS0); // AVCC with external capacitor at AREF pin

   for(div = (F_CPU / 200000L) -1; div > 0; div >>= 1, prescale++)
      {;}
   
   ADCSRA |= prescale;   
}


void AdcDoConversions(void)
{
   uint16_t adc_value;
// NUMBER_OF_INSERTION_CHARACTERS plus one for '\0'
// plus another for a possible - sign, just to be real safe   
   char temp_buf[NUMBER_OF_INSERTION_CHARACTERS + 2];
   uint8_t count;


   ADC_ON;
   
   ADC_SELECT_CHANNEL0;
   adc_value = process_voltage_measurement();
   
// place adc_value, which is an INT into ADC_0
// and convert to ASCII characters
   itoa(adc_value, temp_buf, 10);
   
   count = strlen(temp_buf); // count = number of digits in adc_value
   memcpy((char*)ADC_data_ptr->ADC_0, (char*)temp_buf, sizeof temp_buf);
//   memcpy(ADC_data_ptr->ADC_0, temp_buf, count);

// insert a decimal point  
   ADC_data_ptr->ADC_0[count] = ADC_data_ptr->ADC_0[count - 1];
   ADC_data_ptr->ADC_0[count - 1] = '.';
   
   count++; // to allow for the decimal point
   ADC_data_ptr->count[0] = count;
   

//   ADC_SELECT_CHANNEL1;   
   adc_value = 60;

   itoa(adc_value, temp_buf, 10);
   
   count = strlen(temp_buf);
   memcpy((char*)ADC_data_ptr->ADC_1, (char*)temp_buf, sizeof temp_buf);
//   memcpy(ADC_data_ptr->ADC_1, temp_buf, count);

// insert a decimal point  
   ADC_data_ptr->ADC_1[count] = ADC_data_ptr->ADC_1[count - 1];
   ADC_data_ptr->ADC_1[count - 1] = '.';
   
   count++; // to allow for the decimal point
   ADC_data_ptr->count[1] = count;
   

//   ADC_SELECT_CHANNEL2;   
   adc_value = 105;

   itoa(adc_value, temp_buf, 10);

   count = strlen(temp_buf);
   memcpy((char*)ADC_data_ptr->ADC_2, (char*)temp_buf, sizeof temp_buf);
//   memcpy(ADC_data_ptr->ADC_2, temp_buf, count);

// insert a decimal point  
   ADC_data_ptr->ADC_2[count] = ADC_data_ptr->ADC_2[count - 1];
   ADC_data_ptr->ADC_2[count - 1] = '.';
   
   count++; // to allow for the decimal point
   ADC_data_ptr->count[2] = count;


   ADC_SELECT_CHANNEL3;  
   adc_value = process_temperature_measurement();

   itoa(adc_value, temp_buf, 10);

   count = strlen(temp_buf);
   memcpy((char*)ADC_data_ptr->ADC_3, (char*)temp_buf, sizeof temp_buf);
//   memcpy(ADC_data_ptr->ADC_3, temp_buf, count);
   ADC_data_ptr->count[3] = count;
  
  // insert a decimal point  
   ADC_data_ptr->ADC_3[count] = ADC_data_ptr->ADC_3[count - 1];
   ADC_data_ptr->ADC_3[count - 1] = '.';
   
   count++; // to allow for the decimal point
   ADC_data_ptr->count[3] = count;
   

//   ADC_SELECT_CHANNEL4; 
   adc_value = 320;

   itoa(adc_value, temp_buf, 10);

   count = strlen(temp_buf);
   memcpy((char*)ADC_data_ptr->ADC_4, (char*)temp_buf, sizeof temp_buf);
//   memcpy(ADC_data_ptr->ADC_4, temp_buf, count);
  
// insert a decimal point  
   ADC_data_ptr->ADC_4[count] = ADC_data_ptr->ADC_4[count - 1];
   ADC_data_ptr->ADC_4[count - 1] = '.';
   
   count++; // to allow for the decimal point
   ADC_data_ptr->count[4] = count;
 
   
   ADC_OFF;
}


// ******************************************************************
//
// Function Name:    get_ADC_data    
// Description:        
// Preconditions:     
// Input Parameters: pointer to a struct in httpd.c  
// Return Parameter: 
// Effects:           
//
// Notes: 
//
// ******************************************************************

void get_ADC_data(const ADC_data_s *ADC_data_get_ptr)
{
  (void) memcpy((void *)ADC_data_get_ptr, (void *)&ADC_data, sizeof(ADC_data));
}
/* ---------- end of get_ADC_data() ---------- */



// ******************************************************************
//
// Function Name: process_temperature_measurement();   
// Description:        
// Preconditions:     
// Input Parameters: None 
// Return Parameter: None 
// Effects:           
//
// ******************************************************************
static uint16_t process_temperature_measurement(void)
{
   uint16_t temperature_input = 0;
   uint16_t raw_temperature_value = 0;
  
// discarding the first conversion seems to be a common approach
   ADCSRA |= (1<<ADSC);       // start a/d conversion
   while(ADCSRA & (1<<ADSC)); // wait while ADSC bit is high

   ADCSRA |= (1<<ADSC);       // start a/d conversion
   while(ADCSRA & (1<<ADSC)); // wait while ADSC bit is high

   raw_temperature_value = ADCW; // 10 bits in 2 registers

/* Using a gain of 1 for the LM35, 100 degrees is 1.0V, which is an ADC
   reading of ((1.0 * 1024) / 5.0) or 205. Therefore, just divide the
   ADC reading by 2 to get a direct readout in degrees C.
   Use 10/21 to get the result a bit closer.
   
   Max millivolts out of the LM35 = 150 C * 10mV/C = 1.5Volts
   This equals 1.5/5.0 * 1024 bits (for a 10 bit ADC) = 307 bits.
   
   Multiply by 10 to shift digits left one place.  Also multiply
   trap value by 10
*/   

   temperature_input = ((raw_temperature_value * 100) / 21);

   if(temperature_input > 1460) // (307 (max useable LM35 value) * 100 / 21)
   {
      temperature_input = 1460; // traps any value over 146.0 degrees
   }
   else
      {;}

   return temperature_input; 
}
/* ---------- end of process_temperature measurement() ---------- */


// ******************************************************************
//
// Function Name: process_voltage_measurement();   
// Description:        
// Preconditions:     
// Input Parameters: None 
// Return Parameter: None 
// Effects:           
//
// ******************************************************************
static uint16_t process_voltage_measurement(void)
{
   uint16_t voltage_input = 0;
   uint16_t raw_voltage_value = 0;
  

// discarding the first conversion seems to be a common approach
   ADCSRA |= (1<<ADSC);       // start a/d conversion
   while(ADCSRA & (1<<ADSC)); // wait while ADSC bit is high

   ADCSRA |= (1<<ADSC);       // start a/d conversion
   while(ADCSRA & (1<<ADSC)); // wait while ADSC bit is high


   raw_voltage_value = ADCW;   // 10 bits in 2 registers

/*
  10K to ground and 22K to sensed voltage
  1024 bits equals 16.0 Volts 
*/
   voltage_input = ((raw_voltage_value * 20) / 130);

   if(voltage_input > 160)
   {
      voltage_input = 160; // traps any value over 16.0Volts
   }
   else
      {;}

   return voltage_input; 
}
/* ---------- end of process_voltage measurement() ---------- */
