//*****************************************************************************
//
// File Name	: buffer.c
// Title	   	: Multipurpose byte buffer structure and methods
// Author		: Pascal Stang - Copyright (C) 2001-2002
// Created		: 9/23/2001
// Revised		: D. Festing 14/7/2008
// Version		: 1.0
// Target MCU	: ATmega32
// Editor Tabs	: 
//
// This code is distributed under the GNU Public License
//	which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************


#include "project.h" // has to be the first header file

#include "buffer.h"


void bufferInit(cBuffer* buffer, uint8_t *start, uint16_t size)
{
   ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
   {
// set start pointer of the buffer
      buffer->dataptr = start;
      buffer->size = size;

      buffer->dataindex = 0;
      buffer->datalength = 0;
   }
}

uint8_t bufferGetFromFront(cBuffer* buffer)
{
   uint8_t data = 0;


   ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
   {  
	
   // check to see if there's data in the buffer
      if(buffer->datalength)
      {
		// get the first character from buffer
         data = buffer->dataptr[buffer->dataindex];

		// move index down and decrement length
         buffer->dataindex++;
      
         if(buffer->dataindex >= buffer->size)
         {
            buffer->dataindex -= buffer->size;
         }
   
         buffer->datalength--;
	   }
      else
         {;}
   }  

   return data;
}

bool_t bufferAddToEnd(cBuffer* buffer, uint8_t data)
{
   bool_t status = FALSE;
   

   ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
   {   
      if(buffer->datalength < buffer->size) // make sure the buffer has room
      {
      // save data byte at end of buffer
         buffer->dataptr[(buffer->dataindex + buffer->datalength) % buffer->size] = data;

         buffer->datalength++;

         status = TRUE;
      }
      else
      {
         status = FALSE;
      }
   }
   
   return status;
}

