/* Name: dac.c
   Purpose: To initialise and load a Mitsubishi M62354P 8 bit 6 channel DAC
   Author: Dave Festing
   Date: 9 May 2009
*/


/* Notes:
- N.B. bytes stored in memory MSB first
- reverse_bits() to format address correctly for the DAC, the DAC selection
  (address) bits
- if you try to reverse all 8 bits then the lower (wanted) 4 bits get
  shifted out of the shift register AND lost
- M62354, CPOL = 0 and CPHA = 0  
*/


#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <util/delay.h>
#include "dac.h"
#include "spi.h"


static DAC_data_s DAC_data;                   // define the DAC_data structure
static DAC_data_s *DAC_data_ptr = &DAC_data; // define/declare and initialise a pointer to DAC_data    


static uint8_t reverse_4bits(uint8_t bits_to_reverse, int8_t number_of_bits);


void load_DAC(uint8_t address, uint8_t value)
{
   char temp_buf[4];
   uint8_t count;

   
// place the parsed DAC value into the DAC_data struct  
   itoa(value, temp_buf, 10);

   count = strlen(temp_buf); // count = number of digits in adc_value

// put two zeros in front if less than 10
   if(value < 10)
   {
      temp_buf[2] = temp_buf[0];
      temp_buf[1] = '0';
      temp_buf[0] = '0';
      count = count + 2;
   }
// put one zero in front if less than 100
   else if(value < 100)
   {
      temp_buf[2] = temp_buf[1];
      temp_buf[1] = temp_buf[0];
      temp_buf[0] = '0';
      count++;
   }

   switch (address)
   {
      case 0:  break;

      case 1:  memcpy((char*)DAC_data_ptr->DAC_1, (char*)temp_buf, sizeof temp_buf);
            //   memcpy(DAC_data_set_ptr->DAC_1, temp_buf, temp_buf);
               DAC_data_ptr->count[1] = count;

               break;

      case 2:  memcpy((char*)DAC_data_ptr->DAC_2, (char*)temp_buf, sizeof temp_buf);
            //   memcpy(DAC_data_set_ptr->DAC_2, temp_buf, temp_buf);
               DAC_data_ptr->count[2] = count;

               break;

      case 3:  memcpy((char*)DAC_data_ptr->DAC_3, (char*)temp_buf, sizeof temp_buf);
            //   memcpy(DAC_data_set_ptr->DAC_3, temp_buf, temp_buf);
               DAC_data_ptr->count[3] = count;

               break;

      case 4:  memcpy((char*)DAC_data_ptr->DAC_4, (char*)temp_buf, sizeof temp_buf);
            //   memcpy(DAC_data_set_ptr->DAC_4, temp_buf, temp_buf);
               DAC_data_ptr->count[4] = count;

               break;

      default:
               break;
   }         

   address = reverse_4bits(address, 4);
   
// slow the SPI down
//   SPCR |= (1<<SPR0)|(1<<SPR1);
//   SPSR |= (0<<SPI2X);
  
   DAC_DESELECT(); // ensure DAC_enable is low
  
   SpiWriteByte(address);
   SpiWriteByte(value);
 
   DAC_SELECT();
   
//   _NOP(); // one NOP at 12.5MHz should be about 100ns
             // but didn't work
   
   _delay_ms(1); // LD pulse has to be wider than 100ns
 
   DAC_DESELECT(); // takes 100ns to complete at 16MHz

// restore full SPI speed
//   SPCR |= (0<<SPR0)|(0<<SPR1);
//   SPSR |= (1<<SPI2X);
}
/* -------------- end of load_DAC -------------- */


static uint8_t reverse_4bits(uint8_t bits_to_reverse, int8_t number_of_bits)
{
   uint8_t reversed_bits = 0;
// number_of_bits a signed variable as it used as a loop counter
   
   do
   {
      reversed_bits |= (uint8_t)(bits_to_reverse & 1);
      bits_to_reverse >>= 1;
      reversed_bits <<= 1;
   }  while(--number_of_bits  >  0);
    
   return (uint8_t)(reversed_bits >> 1);
}
/* -------------- end of reverse_bits -------------- */


// ******************************************************************
//
// Function Name:    get_DAC_data    
// Description:        
// Preconditions:     
// Input Parameters: pointer to a struct in http.c 
// Return Parameter: 
// Effects:           
//
// Notes: 
//
// ******************************************************************

void get_DAC_data(const DAC_data_s *DAC_data_get_ptr)
{
  (void) memcpy((void *)DAC_data_get_ptr, (void *)&DAC_data, sizeof(DAC_data));
}
/* ---------- end of get_DAC_data() ---------- */


// ******************************************************************
//
// Function Name:    set_DAC_data    
// Description:        
// Preconditions:     
// Input Parameters: pointer to a struct in main.c 
// Return Parameter: 
// Effects:           
//
// Notes: 
//
// ******************************************************************

void set_DAC_data(const DAC_data_s *DAC_data_set_ptr)
{
  (void) memcpy((void *)&DAC_data, (void *)DAC_data_set_ptr, sizeof(DAC_data));
}
/* ---------- end of set_DAC_data() ---------- */
