// ******************************************************************
//
// File Name   : usart.c
// Author      : Pascal Stang - Copyright (C) 2000-2002
// Created     : 11/22/2000
// Revised     : 06/09/2003
// Revised     : D. Festing 4/7/2008
// Version     : 1.0
// Target MCU  : Atmel AVR series
// Editor Tabs	: 3
//
// Description : This file contains the functions that allow USART
//               communications
//
// ******************************************************************


#include "project.h"
#include "usart.h"

volatile uint8_t usartReadyTx = 0; // flag    
cBuffer usartRxBuffer;         // usart receive buffer
// no TX buffer
uint16_t usartRxOverflow = 0;      // receive overflow counter
// allocate space in ram for RX buffer
static uint8_t usartRxData[USART_RX_BUFFER_SIZE] = {0};


void usartInit(void)
{
   bufferInit(&usartRxBuffer, usartRxData, USART_RX_BUFFER_SIZE);

   usartReadyTx = TRUE; // start in the ready condition

   usartRxOverflow = 0; // clear overflow count
}

// transmits a byte over the usart
void usartSendByte(uint8_t txData)
{
// wait for the transmitter to be ready
   while(usartReadyTx != TRUE)
      {;}

   UDR = txData; // send byte

   usartReadyTx = FALSE; // set ready state
}

// gets a byte (if available) from the usart receive buffer
uint8_t usartReceiveByte(uint8_t* rxData)
{
// make sure we have a receive buffer
   if(usartRxBuffer.size)
   {
   // make sure we have data
      if(usartRxBuffer.datalength)
      {
      // get byte from beginning of buffer
         *rxData = bufferGetFromFront(&usartRxBuffer);
         return TRUE;
      }
      else
      {
         return FALSE; // no data
      }
   }
   else
   {
      return FALSE; // no buffer
   }
}


// USART RX Complete Interrupt Handler
USART_INTERRUPT_HANDLER(USART_RXC_vect)
{
   uint8_t received_char = 0;
   bool_t ret_val = FALSE;
 
  
   received_char = UDR;

   ret_val = bufferAddToEnd(&usartRxBuffer, received_char);
   
   if(ret_val == FALSE)
   {
      usartRxOverflow++;
   }
   else
      {;} // everything is OK   
}

// USART TX Complete Interrupt Handler
USART_INTERRUPT_HANDLER(USART_TXC_vect)
{
// Tx complete, back to ready
   usartReadyTx = TRUE;
}

